/***
 * Excerpted from "Pragmatic Guide to Subversion",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit http://www.pragmaticprogrammer.com/titles/pg_svn for more book information.
***/
import com.mongodb.DB;
import com.mongodb.Mongo;
import com.mongodb.MongoOptions;
import util.populator;
import util.worker;

import java.net.UnknownHostException;

public class mbench {
    public static void main(String[] args) throws UnknownHostException {
        if (args.length != 3) {
            usage();
            return;
        }

        String dbHost = args[0];
        long docCount = Long.parseLong(args[1]);
        int runTime = Integer.parseInt(args[2]);

        MongoOptions options = new MongoOptions();
        options.connectTimeout = 10000;
        options.socketTimeout = 10000;

        Mongo m = new Mongo(dbHost, options);

        System.out.println("Dropping performance_test database...");
        m.dropDatabase("performance_test");

        DB db = m.getDB("performance_test");

        System.out.println("Populating database with " + docCount + " documents...");
        populator pop = new populator(db, docCount);
        pop.populateDb();

        System.out.println("Exercising database for " + runTime + " seconds...");
        worker worker = new worker(db, runTime);
        worker.go();

        int readsPerSec = (int) (worker.getReadsCompleted() / runTime);
        int writesPerSec = (int) (worker.getWritesCompleted() / runTime);

        System.out.println("Reads/sec:  " + readsPerSec);
        System.out.println("Writes/sec: " + writesPerSec);
    }

    private static void usage() {
        System.out.println("Usage: mbench <db host> <# documents> <run time>");
    }
}
