/***
 * Excerpted from "Pragmatic Guide to Subversion",
 * published by The Pragmatic Bookshelf.
 * Copyrights apply to this code. It may not be used to create training material,
 * courses, books, articles, and the like. Contact us if you are in doubt.
 * We make no guarantees that this code is fit for any purpose.
 * Visit http://www.pragmaticprogrammer.com/titles/pg_svn for more book information.
***/
package util;

import com.mongodb.*;

import java.util.Calendar;
import java.util.Date;
import java.util.HashSet;
import java.util.Random;

public class worker {

    private final DB db;
    private final int runTime;

    private long readsCompleted = 0;
    private long writesCompleted = 0;

    public worker(DB db, int runTime) {
        this.db = db;
        this.runTime = runTime;
    }

    public long getReadsCompleted() {
        return readsCompleted;
    }

    public long getWritesCompleted() {
        return writesCompleted;
    }

    public void go() {

        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.SECOND, runTime);
        Date finish = cal.getTime();

        DBCollection collection = db.getCollection("test_collection");

        DBCursor dbCursor = collection.find();

        HashSet<Object> ids = new HashSet<Object>();

        while (dbCursor.hasNext()) {
            DBObject item = dbCursor.next();
            Object id = item.get("_id");
            ids.add(id);
        }

        Object[] idArray = ids.toArray();
        Random r = new Random();

        while (new Date().before(finish)) {
            Object id = idArray[r.nextInt(idArray.length)];
            GetById(collection, id);
            readsCompleted++;

            if (r.nextInt(100) < 10) {
                DBObject item = GetById(collection, id);
                item.put("some_id", r.nextInt(50000));
                writesCompleted++;
            }
        }
    }

    private static DBObject GetById(DBCollection inventory, Object id) {
        for (int attempt = 1; attempt <= 10; attempt++) {
            try {
                return TryGetById(inventory, id);
            } catch (Exception e) {
                System.out.println("Error on attempt " + attempt + ": " + e.getMessage());
            }
        }
        throw new RuntimeException("Failed to get Mongo object even after retries.");
    }

    private static DBObject TryGetById(DBCollection inventory, Object id) {
        BasicDBObject query = new BasicDBObject();
        query.put("_id", id);
        DBCursor cursor = inventory.find(query);
        if (!cursor.hasNext()) {
            System.out.println("Couldn't find object with _id " + id);
            return null;
        }
        return cursor.next();
    }
}
